package fr.jcreigno.blazeds;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import flex.messaging.Destination;
import flex.messaging.config.ConfigMap;

/**
 * Content handler impl to parse remoting.
 * @author jcreigno
 */
final class DefaultHandlerExtension extends DefaultHandler {

    /**
     * wiringService.
     */
    private final AutowiringBootstrapService wiringService;

    /**
     * @param autowiringBootstrapService
     */
    public DefaultHandlerExtension(AutowiringBootstrapService autowiringBootstrapService) {
        this.wiringService = autowiringBootstrapService;
    }

    /** ID attribute. */
    private static final String ID_ATTR = "id";

    /** Properties tag. */
    private static final String PROPERTIES = "properties";

    /** Destination tag. */
    private static final String DESTINATION = "destination";

    /** result. */
    List<Destination> result = new ArrayList<Destination>();

    /** Current destination id. */
    private String id = null;

    /** Current destination. */
    private Destination destination = null;

    /** Current destination config. */
    private final ConfigMap config = new ConfigMap();

    /** Current tag. */
    private String tag = null;

    /** read chars. */
    private StringBuilder characters = null;

    /**
     * {@inheritDoc}
     */
    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (characters != null) {
            characters.append(new String(ch, start, length));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void endElement(String uri, String localName, String name) throws SAXException {
        if (DESTINATION.equals(name)) {
            result.add(destination);
        } else if (PROPERTIES.equals(name)) {
            destination.initialize(id, config);
            config.clear();
        } else {
            config.addProperty(tag, characters.toString());
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
        if (DESTINATION.equals(name)) {
            id = attributes.getValue(ID_ATTR);
            destination = this.wiringService.service.createDestination(id);
        } else if (!PROPERTIES.equals(name)) {
            tag = name;
            characters = new StringBuilder();
        }
    }
}